@@ -4,12 +4,6 @@
 #include "PF/densities.h"
 #include "PF/est_params.h"
 
-#define BOOT_FILTER "bootstrap_filter"
-#define PF_APPROX_CLOUD_MEAN "PF_normal_approx_w_cloud_mean"
-#define AUX_APPROX_CLOUD_MEAN "AUX_normal_approx_w_cloud_mean"
-#define PF_APPROX_PARTICLE "PF_normal_approx_w_particles"
-#define AUX_APPROX_PARTICLE "AUX_normal_approx_w_particles"
-
 /* --------------------------------------- */
 
 template<
@@ -46,7 +40,7 @@ class PF_smooth_smoother_n_dens {
 public:
   static Rcpp::List compute(
       const PF_data &data, const std::string method,
-      pf_base_dens &dens_calc){
+      pf_dens &dens_calc){
     /* Get the smoothed particles at time 1, 2, ..., d */
     smoother_output result;
     if (method == BOOT_FILTER) {
@@ -85,7 +79,7 @@ class PF_smooth_dens {
 public:
   static Rcpp::List compute(
       const PF_data &data, const std::string smoother,
-      const std::string method, pf_base_dens &dens_calc){
+      const std::string method, pf_dens &dens_calc){
     Rcpp::List ans;
 
     if(smoother == "Fearnhead_O_N"){
@@ -113,8 +107,8 @@ Rcpp::List PF_smooth(
     const arma::mat &R, arma::mat &Q_0,
     arma::mat &Q, const arma::mat Q_tilde, const Rcpp::List &risk_obj,
     const arma::mat &F, const int n_max, const int n_threads,
-    const arma::vec &fixed_parems, const int N_fw_n_bw, const int N_smooth,
-    const int N_smooth_final,
+    const arma::vec &fixed_params, const int N_fw_n_bw, const int N_smooth,
+    const int N_smooth_final, const double covar_fac, const double ftol_rel,
     Rcpp::Nullable<Rcpp::NumericVector> forward_backward_ESS_threshold,
     const int debug, const int N_first, std::string type, const int nu,
 
@@ -122,47 +116,18 @@ Rcpp::List PF_smooth(
     const std::string method, const std::string smoother, const std::string model){
   const arma::ivec is_event_in_bin = Rcpp::as<arma::ivec>(risk_obj["is_event_in"]);
 
-  std::unique_ptr<PF_data> data;
-  if(type == "RW")
-    data.reset(new random_walk<PF_data>(
-        n_fixed_terms_in_state_vec,
-        X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
-        risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, N_smooth_final, forward_backward_ESS_threshold, debug,
-        N_first, nu));
-  else if (type == "VAR")
-    data.reset(new PF_data(
-        n_fixed_terms_in_state_vec,
-        X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
-        risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, N_smooth_final, forward_backward_ESS_threshold, debug,
-        N_first, nu));
-  else
-    Rcpp::stop("'type' not implemented");
+  std::unique_ptr<PF_data> data(new PF_data(
+      n_fixed_terms_in_state_vec,
+      X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
+      risk_obj, F, n_max, n_threads, fixed_params, Q_tilde,
+      N_fw_n_bw, N_smooth, N_smooth_final, forward_backward_ESS_threshold,
+      debug, N_first, nu, covar_fac, ftol_rel));
 
   Rcpp::List ans;
-
-  std::unique_ptr<pf_base_dens> dens_calc;
-  if(model == "logit"){
-    dens_calc.reset(new logistic_dens(*data.get()));
-
-  } else if (model == "exponential"){
-    dens_calc.reset(new exponential_dens(*data.get()));
-
-  } else if (model == "cloglog") {
-    dens_calc.reset(new cloglog_dens(*data.get()));
-
-  } else {
-    std::stringstream stream;
-    stream << "model '" << model << "' is not implemented";
-    Rcpp::stop(stream.str());
-
-  }
-
-  ans = PF_smooth_dens::compute(
-    *data.get(), smoother, method, *dens_calc.get());
-
-  return(ans);
+  std::string family_use = get_family(model);
+  pf_dens dens_calc(*data, family_use);
+  return PF_smooth_dens::compute(
+    *data.get(), smoother, method, dens_calc);
 }
 
 /* --------------------------------------- */
@@ -203,7 +168,7 @@ class PF_single_direction {
 public:
   static Rcpp::List compute(
       const PF_data &data, const std::string method,
-      pf_base_dens &dens_calc){
+      pf_dens &dens_calc){
     /* Get the smoothed particles at time 1, 2, ..., d */
     std::vector<cloud> result;
     if (method == BOOT_FILTER) {
@@ -235,7 +200,7 @@ public:
 
 Rcpp::List PF_single_direction_compute(
     const PF_data &data, const bool is_forward, const std::string method,
-    pf_base_dens &dens_calc){
+    pf_dens &dens_calc){
   if(is_forward)
     return PF_single_direction<true>::compute(data, method, dens_calc);
 
@@ -249,54 +214,26 @@ Rcpp::List particle_filter(
     const arma::mat &R, arma::mat &Q_0,
     arma::mat &Q, const arma::mat Q_tilde, const Rcpp::List &risk_obj,
     const arma::mat &F, const int n_threads,
-    const arma::vec &fixed_parems, const int N_fw_n_bw,
+    const arma::vec &fixed_params, const int N_fw_n_bw,
     Rcpp::Nullable<Rcpp::NumericVector> forward_backward_ESS_threshold,
     const int debug, const int N_first, const int nu, std::string type,
-    const bool is_forward, const std::string method, const std::string model){
+    const bool is_forward, const std::string method, const std::string model,
+    const double covar_fac, const double ftol_rel){
   const arma::ivec is_event_in_bin =
     Rcpp::as<arma::ivec>(risk_obj["is_event_in"]);
 
-  std::unique_ptr<PF_data> data;
   const unsigned int N_smooth = 1, n_max = 1, N_smooth_final = 1;
-  if(type == "RW")
-    data.reset(new random_walk<PF_data>(
-        n_fixed_terms_in_state_vec,
-        X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
-        risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, N_smooth_final,
-        forward_backward_ESS_threshold, debug, N_first, nu));
-  else if (type == "VAR")
-    data.reset(new PF_data(
-        n_fixed_terms_in_state_vec,
-        X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
-        risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, N_smooth_final,
-        forward_backward_ESS_threshold, debug, N_first, nu));
-  else
-    Rcpp::stop("'type' not implemented");
+  std::unique_ptr<PF_data> data(new PF_data(
+      n_fixed_terms_in_state_vec,
+      X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
+      risk_obj, F, n_max, n_threads, fixed_params, Q_tilde,
+      N_fw_n_bw, N_smooth, N_smooth_final, forward_backward_ESS_threshold,
+      debug, N_first, nu, covar_fac, ftol_rel));
 
-  Rcpp::List ans;
-
-  std::unique_ptr<pf_base_dens> dens_calc;
-  if(model == "logit"){
-    dens_calc.reset(new logistic_dens(*data.get()));
-
-  } else if (model == "exponential"){
-    dens_calc.reset(new exponential_dens(*data.get()));
-
-  } else if (model == "cloglog") {
-    dens_calc.reset(new cloglog_dens(*data.get()));
-
-  } else {
-    std::stringstream stream;
-    stream << "model '" << model << "' is not implemented";
-    Rcpp::stop(stream.str());
-
-  }
-
-  ans = PF_single_direction_compute(*data.get(), is_forward, method, *dens_calc.get());
-
-  return(ans);
+  std::string family_use = get_family(model);
+  pf_dens dens_calc(*data, family_use);
+  return PF_single_direction_compute(
+    *data.get(), is_forward, method, dens_calc);
 }
 
 // [[Rcpp::export]]
@@ -355,3 +292,92 @@ Rcpp::List PF_est_params_dens(
     Rcpp::Named("QR_F")    = new_params.F,
     Rcpp::Named("QR_dev")  = new_params.dev);
 }
+
+
+
+// [[Rcpp::export]]
+Rcpp::List PF_get_score_n_hess_cpp(
+  const Rcpp::List fw_cloud, arma::mat &Q,
+  const arma::mat &F, Rcpp::List risk_obj,
+  arma::mat &ran_vars, arma::mat &fixed_terms,
+  const arma::vec &tstart, const arma::vec &tstop,
+  const arma::vec &fixed_params, const std::string family,
+  const int max_threads, const bool debug, const arma::vec &a_0,
+  const arma::mat &R, arma::mat &Q_0, const arma::mat &Q_tilde,
+  const arma::uword N_fw_n_bw, const arma::uword N_first, const double nu,
+  const double covar_fac, const double ftol_rel, const std::string method,
+  Rcpp::Nullable<Rcpp::NumericVector> forward_backward_ESS_threshold,
+  const bool use_O_n_sq = false, const bool only_score = false)
+{
+  std::vector<cloud> clouds_cpp;
+  const Rcpp::List risk_sets_R = Rcpp::as<Rcpp::List>(risk_obj["risk_sets"]);
+  std::vector<arma::uvec> risk_sets(risk_sets_R.size());
+  for(unsigned int i = 0; i < risk_sets_R.size(); ++i)
+    risk_sets[i] = Rcpp::as<arma::uvec>(risk_sets_R[i]) - 1L;
+
+  arma::ivec is_event_in = Rcpp::as<arma::ivec>(risk_obj["is_event_in"]);
+  arma::vec event_times = Rcpp::as<arma::vec>(risk_obj["event_times"]);
+
+  std::vector<std::unique_ptr<score_n_hess_base> > out_cpp;
+  if(use_O_n_sq){
+    out_cpp = PF_get_score_n_hess_O_N_sq
+    (Q, F, risk_sets, risk_obj, is_event_in, event_times, ran_vars,
+     fixed_terms, tstart, tstop, fixed_params, family, max_threads, debug,
+     only_score, a_0, R, Q_0, Q_tilde, N_fw_n_bw, N_first, nu, covar_fac,
+     ftol_rel, forward_backward_ESS_threshold, method);
+
+  } else {
+    clouds_cpp = get_cloud_from_rcpp_list<false, false>(fw_cloud);
+    out_cpp = PF_get_score_n_hess
+      (clouds_cpp, Q, F, risk_sets, is_event_in, event_times, ran_vars,
+       fixed_terms, tstart, tstop, fixed_params, family, max_threads, debug,
+       only_score);
+  }
+
+  arma::uword
+    p = out_cpp[0]->get_a_state().n_elem, q = out_cpp[0]->get_a_obs().n_elem;
+  int i_p = p, i_q = q;
+  arma::vec S_state(p, arma::fill::zeros), S_obs(q, arma::fill::zeros);
+  arma::mat neg_obs_info_state_dd(p, p, arma::fill::zeros),
+            neg_obs_info_state_d (p, p, arma::fill::zeros),
+            neg_obs_info_obs_dd  (q, q, arma::fill::zeros),
+            neg_obs_info_obs_d   (q, q, arma::fill::zeros);
+
+  for(auto &o : out_cpp){
+    const double w = o->get_weight();
+    S_state += w * o->get_a_state();
+    S_obs   += w * o->get_a_obs();
+
+    if(!only_score){
+      R_BLAS_LAPACK::sym_mat_rank_one_update
+        (&i_p, &w, o->get_a_state().memptr(), neg_obs_info_state_d.memptr());
+      R_BLAS_LAPACK::sym_mat_rank_one_update
+        (&i_q, &w, o->get_a_obs()  .memptr(), neg_obs_info_obs_d  .memptr());
+
+      neg_obs_info_state_dd += w * o->get_B_state();
+      neg_obs_info_obs_dd   += w * o->get_B_obs();
+    }
+  }
+
+  if(!only_score){
+    neg_obs_info_state_d    = arma::symmatu(neg_obs_info_state_d);
+    neg_obs_info_state_dd   = arma::symmatu(neg_obs_info_state_dd);
+    neg_obs_info_obs_d      = arma::symmatu(neg_obs_info_obs_d);
+    neg_obs_info_obs_dd     = arma::symmatu(neg_obs_info_obs_dd);
+
+  } else {
+    neg_obs_info_state_d.fill(NA_REAL);
+    neg_obs_info_state_dd.fill(NA_REAL);
+    neg_obs_info_obs_d.fill(NA_REAL);
+    neg_obs_info_obs_dd.fill(NA_REAL);
+
+  }
+
+  return Rcpp::List::create(
+    Rcpp::Named("S_state")              = std::move(S_state),
+    Rcpp::Named("S_obs")                = std::move(S_obs),
+    Rcpp::Named("E_second_deriv_state") = std::move(neg_obs_info_state_dd),
+    Rcpp::Named("E_second_deriv_obs")   = std::move(neg_obs_info_obs_dd),
+    Rcpp::Named("E_score_outer_state")  = std::move(neg_obs_info_state_d),
+    Rcpp::Named("E_score_outer_obs")    = std::move(neg_obs_info_obs_d));
+}
