@@ -3,6 +3,40 @@
 
 #include "../arma_n_rcpp.h"
 
+class particle;
+
+class cloud : private std::vector<particle> {
+  using Base = std::vector<particle>;
+
+public:
+  using Base::size_type;
+
+  cloud(): Base() {}
+  cloud(size_type count): Base(count) {}
+
+  void new_particle(
+      arma::vec state, const particle *parent = nullptr, const particle *child = nullptr);
+  void new_particle(particle&&);
+  particle& set_particle(
+      arma::uword idx, arma::vec state,
+      const particle *parent = nullptr, const particle *child = nullptr);
+  arma::vec get_weigthed_mean();
+
+  using Base::begin;
+  using Base::end;
+  using Base::rbegin;
+  using Base::rend;
+  using Base::iterator;
+  using Base::const_iterator;
+  using Base::reverse_iterator;
+  using Base::value_type;
+  using Base::const_reverse_iterator;
+  using Base::back;
+  using Base::operator[];
+  using Base::reserve;
+  using Base::size;
+};
+
 class particle {
   arma::vec state;
   arma::uword cloud_idx; /* zero based */
@@ -29,35 +63,8 @@ public:
     const arma::vec state, const particle *parent,
     const arma::uword cloud_idx, const particle *child = nullptr);
   particle();
-};
 
-class cloud : private std::vector<particle> {
-  using Base = std::vector<particle>;
-
-public:
-  using Base::size_type;
-  using Base::iterator;
-  using Base::const_iterator;
-  using Base::reverse_iterator;
-
-  cloud(): Base() {}
-  cloud(size_type count): Base(count) {}
-
-  void new_particle(
-      arma::vec state, const particle *parent = nullptr, const particle *child = nullptr);
-  particle& set_particle(
-      arma::uword idx, arma::vec state,
-      const particle *parent = nullptr, const particle *child = nullptr);
-  arma::vec get_weigthed_mean();
-
-  using Base::begin;
-  using Base::end;
-  using Base::rbegin;
-  using Base::back;
-  using Base::rend;
-  using Base::operator[];
-  using Base::reserve;
-  using Base::size;
+  friend void cloud::new_particle(particle&&);
 };
 
 #endif
